#include "Database.hpp"

Napi::Object Database::Init(Napi::Env env, Napi::Object exports) {
    Napi::Function func =
        DefineClass(env, "Database",
                    {InstanceMethod("loadScript", &Database::LoadScript), InstanceMethod("build", &Database::Build),
                     InstanceMethod("messages", &Database::GetMessages)}
        );

    auto* constructor = new Napi::FunctionReference();
    *constructor = Napi::Persistent(func);

    env.SetInstanceData(constructor);

    exports.Set("Database", func);
    return exports;
}

void Database::MessageCallback(const asSMessageInfo* msg) {
    if (msg->type == asEMsgType::asMSGTYPE_INFORMATION)
        return;
    auto obj = Napi::Object::New(_messages.Env());
    obj.Set("section", Napi::String::New(_messages.Env(), msg->section));
    obj.Set("row", Napi::Number::New(_messages.Env(), msg->row));
    obj.Set("col", Napi::Number::New(_messages.Env(), msg->col));
    obj.Set("type", Napi::Number::New(_messages.Env(), msg->type));
    obj.Set("message", Napi::String::New(_messages.Env(), msg->message));
    _messages[_messages.Length()] = obj;
}

Database::Database(const Napi::CallbackInfo& info) : ObjectWrap(info) {
    _engine = asCreateScriptEngine();
    _engine->SetMessageCallback(asMETHOD(Database, MessageCallback), this, asCALL_THISCALL);
    _builder.StartNewModule(_engine, "Module");
}
void Database::LoadScript(const Napi::CallbackInfo& info) {
    if (info.Length() < 2)
        throw std::logic_error("Not enough arguments");
    auto name = info[0].As<Napi::String>().Utf8Value();
    auto script = info[1].As<Napi::String>().Utf8Value();

    _builder.AddSectionFromMemory(name.c_str(), script.c_str());
}
void Database::Build(const Napi::CallbackInfo& info) {
    _messages = Napi::Array::New(info.Env());
    _builder.BuildModule();
}
Napi::Value Database::GetMessages(const Napi::CallbackInfo&) { return _messages; }
